#version 330

uniform sampler2D color_texture;
uniform sampler2D normals_texture;
uniform sampler2D depth_texture;

uniform mat3 G[9] = mat3[](
	1.0/(2.0*sqrt(2.0)) * mat3(1.0, sqrt(2.0), 1.0, 0.0, 0.0, 0.0, -1.0, -sqrt(2.0), -1.0),
	1.0/(2.0*sqrt(2.0)) * mat3(1.0, 0.0, -1.0, sqrt(2.0), 0.0, -sqrt(2.0), 1.0, 0.0, -1.0),
	1.0/(2.0*sqrt(2.0)) * mat3(0.0, -1.0, sqrt(2.0), 1.0, 0.0, -1.0, -sqrt(2.0), 1.0, 0.0),
	1.0/(2.0*sqrt(2.0)) * mat3(sqrt(2.0), -1.0, 0.0, -1.0, 0.0, 1.0, 0.0, 1.0, -sqrt(2.0)),
	1.0/2.0 * mat3(0.0, 1.0, 0.0, -1.0, 0.0, -1.0, 0.0, 1.0, 0.0),
	1.0/2.0 * mat3(-1.0, 0.0, 1.0, 0.0, 0.0, 0.0, 1.0, 0.0, -1.0),
	1.0/6.0 * mat3(1.0, -2.0, 1.0, -2.0, 4.0, -2.0, 1.0, -2.0, 1.0),
	1.0/6.0 * mat3(-2.0, 1.0, -2.0, 1.0, 4.0, 1.0, -2.0, 1.0, -2.0),
	1.0/3.0 * mat3(1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0)
);

out vec4 fragment_color;

void main(void)
{
	mat3 I;
	float cnv[9];
	float sum = 0;

	for (int i = 0; i < 3; i++)
		for (int j = 0; j < 3; j++) {
			float d_ref = texelFetch(depth_texture, ivec2(gl_FragCoord), 0).r;
			float d = texelFetch(depth_texture, ivec2(gl_FragCoord) + ivec2(i - 1, j - 1), 0).r;
			float diff = d - d_ref;
			sum += abs(diff);
			I[i][j] = diff;
		}
	
	// test if we are looking at a flat surface
	float avg = sum / 9;
	if (avg < 0.00001) {
		fragment_color = vec4(0, 0, 0, 1);
		return;
	}

	
	/* calculate the convolution values for all the masks */
	for (int i = 0; i < 9; i++) {
		float dp3 = dot(G[i][0], I[0]) + dot(G[i][1], I[1]) + dot(G[i][2], I[2]);
		cnv[i] = dp3 * dp3; 
	}

	float M = (cnv[0] + cnv[1]) + (cnv[2] + cnv[3]);
	float S = (cnv[4] + cnv[5]) + (cnv[6] + cnv[7]) + (cnv[8] + M);

	if (abs(sqrt(M / S) - 1) > 0.0001)
		fragment_color = vec4(0, 1, 0, 1);
	else
		fragment_color = vec4(0, 0, 0, 1);
}
