#version 330

uniform sampler2D color_texture;
uniform sampler2D normals_texture;
uniform sampler2D depth_texture;

uniform float near = 0.5;
uniform float far = 0.8;
uniform float very_far = 0.9;

uniform mat3 G_3 = mat3(
	1, 2, 1,
	2, 4, 2,
	1, 2, 4);
	
out vec4 fragment_color;

void main(void)
{
	vec3 diffuse_color;
	vec3 normal;

	ivec2 xy = ivec2(gl_FragCoord);

	float depth = texelFetch(depth_texture, xy, 0).r;

	if (depth < near || depth > far) {
		vec3 result = vec3(0, 0, 0);

		if (depth > very_far) {
			int G_5[25];
			G_5[0] = 1;
			G_5[1] = 4;
			G_5[2] = 7;
			G_5[3] = 4;
			G_5[4] = 1;

			G_5[5] = 4;
			G_5[6] = 16;
			G_5[7] = 26;
			G_5[8] = 16;
			G_5[9] = 4;

			G_5[10] = 7;
			G_5[11] = 26;
			G_5[12] = 41;
			G_5[13] = 26;
			G_5[14] = 7;

			G_5[15] = 4;
			G_5[16] = 16;
			G_5[17] = 26;
			G_5[18] = 16;
			G_5[19] = 4;

			G_5[20] = 1;
			G_5[21] = 4;
			G_5[22] = 7;
			G_5[23] = 4;
			G_5[24] = 1;
			
			vec3 color[25];

			for (int i = 0; i < 5; i++)
				for (int j = 0; j < 5; j++) {
					diffuse_color = texelFetch(color_texture, xy + ivec2(i - 1, j - 1), 0).rgb;
					normal = texelFetch(normals_texture, xy + ivec2(i - 1, j - 1), 0).rgb * 2 - 1;

					float dp = dot(normal, vec3(0, 0, 1));
					color[5 * i + j] = diffuse_color * dp;
				}

			for (int i = 0; i < 5; i++)
				for (int j = 0; j < 5; j++)
					result += color[5 * i + j] * G_5[5 * i + j];

			result /= 273.0;
		} else {
			vec3 color[9];

			for (int i = 0; i < 3; i++)
				for (int j = 0; j < 3; j++) {
					diffuse_color = texelFetch(color_texture, xy + ivec2(i - 1, j - 1), 0).rgb;
					normal = texelFetch(normals_texture, xy + ivec2(i - 1, j - 1), 0).rgb * 2 - 1;

					float dp = dot(normal, vec3(0, 0, 1));
					color[3 * i + j] = diffuse_color * dp;
				}

			for (int i = 0; i < 3; i++)
				for (int j = 0; j < 3; j++)
					result += color[3 * i + j] * G_3[i][j];

			result /= 16.0;
		}

		fragment_color = vec4(clamp(result, 0.2, 1), 1);
	} else {
		diffuse_color = texelFetch(color_texture, xy, 0).rgb;
		normal = texelFetch(normals_texture, xy, 0).rgb * 2 - 1;

		float dp = clamp(dot(normal, vec3(0, 0, 1)), 0.2, 1);
		fragment_color = vec4(diffuse_color * dp, 1);
	}
}
